<?php

namespace Drupal\entity_ref_tab_formatter\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_tab_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_tab_formatter",
 *   label = @Translation("Entity reference tab formatter"),
 *   field_types = {
 *     "entity_reference",
 *     "entity_reference_revisions"
 *   }
 * )
 */
class EntityReferenceTabFormatter extends EntityReferenceFormatterBase {

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * Constructs a EntityReferenceEntityFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LoggerChannelFactoryInterface $logger_factory, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->loggerFactory = $logger_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('logger.factory'),
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    // This formatter is only available for entity types that have a view
    // builder.
    $target_type = $field_definition->getFieldStorageDefinition()->getSetting('target_type');
    return \Drupal::entityTypeManager()->getDefinition($target_type)->hasViewBuilderClass();
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      // Implement default settings.
      'tab_title_view_mode' => '',
      'tab_body_view_mode' => '',
      'style' => '',
      'options' => [],
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Implements the settings form.
    $field_settings = $this->getFieldSettings();
    $target_entity_type_id = $field_settings['target_type'];
    $view_mode_options = $this->entityDisplayRepository->getViewModeOptions($target_entity_type_id);

    $elements['tab_title_view_mode'] = array(
      '#type' => 'select',
      '#options' => $view_mode_options,
      '#title' => $this->t('Select the tab title view mode (or use label)'),
      '#default_value' => $this->getSetting('tab_title_view_mode'),
      '#empty_option' => $this->t('- Use label -'),
    );
    // Convenience: If the entity has a full view mode, suggest it.
    $default_view_mode = isset($view_mode_options['full']) ? 'full' : '';
    $elements['tab_body_view_mode'] = array(
      '#type' => 'select',
      '#options' => $view_mode_options,
      '#title' => $this->t('Select the tab body view mode'),
      '#default_value' => $this->getSetting('tab_body_view_mode') ?: $default_view_mode,
      '#required' => TRUE,
    );
    $elements['style'] = array(
      '#type' => 'radios',
      '#options' => array(
        'tab' => 'Tab',
        'accordion' => 'Accordion',
      ),
      '#title' => $this->t('Display Style'),
      '#default_value' => $this->getSetting('style'),
      '#required' => TRUE,
    );
    $elements['options'] = array(
      '#type' => 'yaml',
      '#title' => $this->t('Options'),
      '#default_value' => $this->getSetting('options'),
      '#description' => $this->t('Enter options in <a href="@yamlUrl">Yaml</a> format. For valid options see <a href="@tabsUrl">jQueryTabs</a> and <a href="@accordionUrl">jQueryAccordion</a> docs',
        ['yamlUrl' => 'https://learnxinyminutes.com/docs/yaml/', '@tabsUrl' => 'https://api.jqueryui.com/tabs/', '@dialogsUrl' => 'https://api.jqueryui.com/accordion/']),
    );
    return $elements + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $view_modes = $this->entityDisplayRepository->getViewModeOptions($this->getFieldSetting('target_type'));

    if ($this->getSetting('tab_title_view_mode')) {
      $view_mode = $this->getSetting('tab_title_view_mode');
      $summary[] = t('Title rendered as @mode', ['@mode' => isset($view_modes[$view_mode]) ? $view_modes[$view_mode] : $view_mode]);
    }
    else {
      $summary[] = t('Title from label');
    }

    $view_mode = $this->getSetting('tab_body_view_mode');
    $summary[] = t('Body rendered as @mode', ['@mode' => isset($view_modes[$view_mode]) ? $view_modes[$view_mode] : $view_mode]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    /** @var \Drupal\Core\Field\EntityReferenceFieldItemListInterface $items */
    $style = $this->getSetting('style');
    $target_entity_type_id = $this->getFieldSettings()['target_type'];
    $view_builder = $this->entityTypeManager->getViewBuilder($target_entity_type_id);

    $tabs = array();
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $target_entity) {
      // The template uses this prefixed with 'tab-' as tab anchor.
      $id = $target_entity->id();
      foreach (['title', 'body'] as $part) {
        $view_mode = $this->getSetting("tab_{$part}_view_mode");
        if ($view_mode) {
          $tabs[$id][$part] = $view_builder->view($target_entity, $view_mode);
        }
        else {
          $tabs[$id][$part] = ['#plain_text' => $target_entity->label()];
        }
      }
    }
    $theme = "entity_ref_{$style}_formatter";
    $uniqueId = Html::getUniqueId($theme);
    $options = $this->getSetting('options');
    return array(
      '#theme' => $theme,
      '#tabs' => $tabs,
      '#id' => $uniqueId,
      '#attached' => [
        'drupalSettings' => [
          'entity_ref_tab_formatter' => [
            $style => [
              $uniqueId => $options,
            ],
          ],
        ],
      ],
    );
  }

}
